<?php
class Installer{
	//config file path
	var $config_file_path = "../content/engine/engine_config.php";
	var $database_file_path = "install.sql";
	
	//proxy
	var $proxy_used = false;
	var $proxy_address = "";
	var $proxy_port = 0;
	var $proxy_requires_auth = false;
	var $proxy_username = "";
	var $proxy_password = "";
	
	//database variables
	var $db_hostname = "localhost";
	var $db_username = "root";
	var $db_password = "";
	var $db_database = "shop";
	
	//required libraries
	var $required_functions = array(
		"mysql" => array("mysql_db_query", "MySQL Functions", "http://www.php.net/mysql"),
		"curl" => array("curl_init", "CURL, Client URL Library Functions", "http://www.php.net/curl"),
		"ob" => array("ob_start", "Output Control Functions", "http://www.php.net/outcontrol"),
		"zlib" => array("gzencode", "Zlib Compression Functions", "http://www.php.net/zlib"),
		"image" => array("imagecreate", "GDLib - Image functions", "http://www.php.net/image")
	);
	
	//license validator data
	var $license_validator_url = "https://www.pinnaclecart.com/checksn.php";
	var $license_number = "";
	
	//returned messages and errors array
	var $messages = array();
	var $errors = array();
	
	//constructor
	function Installer(){
		return $this;
	}
	
	//read data from a config file
	function readConfig(){
		@include_once($this->config_file_path);
		if(defined("DB_HOST")) $this->db_hostname = DB_HOST;
		if(defined("DB_USER")) $this->db_username = DB_USER;
		if(defined("DB_PASSWORD")) $this->db_password = DB_PASSWORD;
		if(defined("DB_NAME")) $this->db_database = DB_NAME;
		if(defined("LICENSE_NUMBER")) $this->license_number = LICENSE_NUMBER;
	}
	
	//write data to a config file
	function writeConfig(){
		$f = @fopen($this->config_file_path, "w");
		if($f){
			fputs($f, "<?php\n");
			fputs($f, "\tdefine('DB_HOST', '".$this->db_hostname."');\n");
			fputs($f, "\tdefine('DB_USER', '".$this->db_username."');\n");
			fputs($f, "\tdefine('DB_PASSWORD', '".$this->db_password."');\n");
			fputs($f, "\tdefine('DB_NAME', '".$this->db_database."');\n");
			fputs($f, "\tdefine('LICENSE_NUMBER', '".$this->license_number."');\n");
			fputs($f, "?>\n");
			fclose($f);
			return true;
		}
		else{
			$this->errors[] = "Cannot write config file. Please check file privileges";
			return false;
		}
	}
	
	//step 1 - check required libraries
	function checkInit(){
		$test_passed = true;
		$this->messages = array();
		$this->errors = array();
		
		foreach($this->required_functions as $key=>$test_data){
			if(!function_exists($test_data[0])){
				$this->errors[] = $test_data[1].=" - Is Not Available";
				$test_passed = false;
			}
			else{
				$this->messages[] = $test_data[1].=" - OK";
			}
		}
		
		if(!is_writable($this->config_file_path)){
			$this->errors[] = "Config file is not writable. Please check write permissions. ".$this->config_file_path;
			$test_passed = false;
		}
		else{
			$this->messages[] = "Config file is writable - OK";
		}
		
		if(!is_file($this->database_file_path)){
			$this->errors[] = "Cannot find file with database structure - ".$this->database_file_path;
			$test_passed = false;
		}
		else{
			$this->messages[] = "Database file exists - OK";
		}
		
		return $test_passed;
	}
	
	//step 2 - check license number
	function checkLicenseNumber($license_number){
		//$this->readConfig();
		
		$test_passed = true;
		$this->messages = array();
		$this->errors = array();
		
		$this->license_number = trim($license_number);
		//build post data string
		$post_data = "ip=".base64_encode($_SERVER['SERVER_ADDR']); // Server IP Address
		$post_data .= "&domain=".base64_encode($_SERVER['SERVER_NAME']); // Host Name
		$post_data .= "&url=".base64_encode($_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']); // URL from where script requesting license info
		$post_data .= "&sn=".base64_encode($license_number)."\n\n"; // and, finally, License Number
		
		//create CURL connection and get data
		$c = curl_init($this->license_validator_url);
		
		if($this->proxy_used){
			curl_setopt($c, CURLOPT_PROXY, $this->proxy_address.":".$this->proxy_port);
			if($this->proxy_requires_auth){
				curl_setopt($c, CURLOPT_PROXYUSERPWD, $this->proxy_username.":".$this->proxy_password);
			}
		}
		
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($c, CURLOPT_VERBOSE, 0);
		curl_setopt($c, CURLOPT_HEADER, 0);
		curl_setopt($c, CURLOPT_POSTFIELDS, $post_data);
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		set_time_limit(3000);
		$data = trim(curl_exec($c));
		
		$curl_info = curl_getinfo($c);
		
		//check for CURL error
		if(curl_errno($c)){
			$this->errors[] = "License Server Connection Error. ".curl_error($c);
			$test_passed = false;
		}
		elseif(!in_array($curl_info["http_code"], array(200, 202))){
			$this->errors[] = "Error connecting to remove server. HTTP response code: ".$curl_info["http_code"];
			$test_passed = false;
		}
		else{
			if($data == ""){
				$this->errors[] = "License Server Response Is Empty.";
				$test_passed = false;
			}
			else{
				list($a, $b, $c) = explode("{{", $data);
				$b = trim($b);
				switch($b){
					case "SERIAL_VALID" : {
						$this->messages[] = "License Number is Valid";
						$this->license_number = $license_number;
						break;
					}
					case "DOMAIN_INVALID" : {
						$this->errors[] = "URL / IP address Invalid. This cart resides within an invalid domain, please contact Customer Support";
						$test_passed = false;
						break;
					}
					case "PACKAGE_CLOSED" : {
						$this->errors[] = "Your Licensing Key is Invalid. Check your Licensing Key, or contact Customer Support.";
						$test_passed = false;
						break;
					}
					case "SERIAL_INVALID" : {
						$this->errors[] = "Your Licensing Key is Invalid. Check your Licensing Key, or contact Customer Support.";
						$test_passed = false;
						break;
					}
					case "SERIAL_CLOSED" : {
						$this->errors[] = "Your Licensing Key is Invalid. Check your Licensing Key, or contact Customer Support.";
						$test_passed = false;						
						break;
					}
				}
			}
		}
		
		if($test_passed){
			$test_passed = $this->writeConfig();
		}
		
		return $test_passed;
	}
	
	//step 3 - check database connection
	function installDatabase($db_hostname, $db_username, $db_password, $db_database){
		
		//$this->readConfig();
		
		$this->db_hostname = $db_hostname;
		$this->db_username = $db_username;
		$this->db_password = $db_password;
		$this->db_database = $db_database;
		
		//if(!$this->writeConfig()){
		//	return false;
		//}
		
		$db_link = @mysql_connect($this->db_hostname, $this->db_username, $this->db_password);
		if($db_link){
			if(@mysql_select_db($this->db_database, $db_link)){
				set_time_limit(10000);
				//read filedump from a file
				$f = @fopen($this->database_file_path, "r");
				if($f){
					$i = 0;
					while($s = fgets($f, 4096)){
						if(trim($s) != ""){
							mysql_query(trim($s), $db_link);
						}
				    	if($i % 50  == 0){
							if(!ini_get('safe_mode')){
								usleep(50);
							}
						}
						$i++;
					}
					fclose($f);
					@mysql_close($db_link);
					$this->messages[] = "Database Installed";
					return true;
				}
				else{
					$this->errors[] = "Cannot open database file. ".$this->database_file_path;
					return false;
				}
			}
			else{
				@mysql_close($db_link);
				$this->errors[] = "Cannot select database '".$this->db_database."'";
				return false;
			}
		}
		else{
			$this->errors[] = "Cannot connect to database host '".$this->db_database."'";
			return false;
		}
	}
	
	
	function setInitialParams($data){
		$this->readConfig();
		$db_link = @mysql_connect($this->db_hostname, $this->db_username, $this->db_password);
		if($db_link){
			if(@mysql_select_db($this->db_database, $db_link)){
				mysql_query("UPDATE admins SET email='".addslashes($data["admin_email"])."', name='".addslashes($data["admin_name"])."', username='".addslashes($data["admin_username"])."', password=MD5('".addslashes($data["admin_password"])."')");
				mysql_query("UPDATE settings SET value='".addslashes($data["site_url_http"])."' WHERE name='GlobalHttpUrl'");
				mysql_query("UPDATE settings SET value='".addslashes($data["site_url_https"])."' WHERE name='GlobalHttpsUrl'");
				mysql_query("UPDATE settings SET value='".addslashes($data["site_server_path"])."' WHERE name='GlobalServerPath'");
				
				if(isset($data["proxy"]["proxy_used"])){
					if($data["proxy"]["proxy_used"]){
						mysql_query("UPDATE settings SET value='YES' WHERE name='ProxyAvailable'");
						mysql_query("UPDATE settings SET value='".addslashes($data["proxy"]["proxy_address"])."' WHERE name='ProxyAddress'");
						mysql_query("UPDATE settings SET value='".addslashes($data["proxy"]["proxy_port"])."' WHERE name='ProxyPort'");
						if(isset($data["proxy"]["proxy_requires_auth"])){
							if($data["proxy"]["proxy_requires_auth"]){
								mysql_query("UPDATE settings SET value='YES' WHERE name='ProxyRequiresAuthorization'");
								mysql_query("UPDATE settings SET value='".addslashes($data["proxy"]["proxy_username"])."' WHERE name='ProxyUsername'");
								mysql_query("UPDATE settings SET value='".addslashes($data["proxy"]["proxy_password"])."' WHERE name='ProxyPassword'");
							}
						}
					}
				}
				return true;
			}
			@mysql_close($db_link);
		}
		return false;
	}
	
	
	function updateParams($data){
		$this->readConfig();
		$db_link = @mysql_connect($this->db_hostname, $this->db_username, $this->db_password);
		if($db_link){
			if(@mysql_select_db($this->db_database, $db_link)){
				mysql_query("UPDATE admins SET rights='all', active='Yes', email='".addslashes($data["admin_email"])."', name='".addslashes($data["admin_name"])."', username='".addslashes($data["admin_username"])."', password=MD5('".addslashes($data["admin_password"])."')");
			}
			@mysql_close($db_link);
			return true;
		}
		return false;
	}
}
?>
